<?php
	/* 
	* Simple DBAL (MySQL, using the Mysqli extension)
	* (c) 2009-2011 PART3
	*/
	
	class Database
	{
		public $res; // Allow raw usage of the object
		public $queries = array();
		public $queries_num;
		
		function __construct()
		{
			global $conf, $dbprefix;
			
			// Create DB connection, and discard db pass.
			$this->res = new mysqli( $conf[ 'dbhost' ], $conf[ 'dbuser' ], $conf[ 'dbpass' ], $conf[ 'dbname' ] );
			if ( mysqli_connect_errno() > 0 )
			{
				die();
			}
			$this->res->set_charset( 'utf8' );
			
			define( 'DBPREFIX', $conf[ 'dbprefix' ] );
			$dbprefix = $conf[ 'dbprefix' ];
			
			// These aren't needed any more
			unset( $conf[ 'dbhost' ] );
			unset( $conf[ 'dbuser' ] );
			unset( $conf[ 'dbpass' ] );
			unset( $conf[ 'dbname' ] );
			unset( $conf[ 'dbprefix' ] );
		}
		
		// Main SQL query function. Accepts an unlimited number of parameters, which are first
		// sanitized using the real_escape_string function and then replaced onto the main query.
		// First parameter is always the query.
		function query( $query )
		{
			$numargs  = func_num_args();
			if ( $numargs > 1 )
			{
				$arg_list = func_get_args();
				$next_arg_list = array();
				for ($i = 1; $i < $numargs; $i++)
				{
					$next_arg_list[] = $this->sanitize( $arg_list[ $i ] );
				}
				$query = vsprintf( $query, $next_arg_list );
			}			
			$this->queries[] = $query;
			$this->queries_num++;
			
			return $this->res->query( $query );
		}
		
		// Array building
		// Idea borrowed from phpBB's dbal::sql_build_array
		function build_array( $type, $assoc_ary )
		{
			if ( !is_array( $assoc_ary ) )
			{
				return false;
			}
			$fields = $values = array();
			
			switch( $type )
			{
				case 'INSERT':
					foreach ( $assoc_ary as $key => $var )
					{
						$fields[] = '`' . $key . '`';
						$values[] = "'" . $this->sanitize( $var ) . "'";
					}
					return '(' . implode( ', ', $fields ) . ') VALUES (' . implode( ', ', $values ) . ')';
				break;
				
				case 'UPDATE':
					foreach ($assoc_ary as $key => $var)
					{
						$values[] = "`$key` = '" . $this->sanitize( $var ) . "'";
					}
					return implode( ', ', $values );
				break;
			}
			
			return false;
		}
		
		// 03-10-2010
		// Find last insert id
		// Last update 17-12-2010
		function get_last_insert_id( $Table ) {
			$result = $this->query( "SELECT LAST_INSERT_ID() as `last_insert_id` FROM `%s`", $Table );
			$row = $result->fetch_object();
			return $row->last_insert_id;
		}
		
		// Basic sanitization using mysqli::real_escape_string
		function sanitize( $var )
		{
			return $this->res->real_escape_string( $var );
		}
	}
?>